import 'package:flutter/material.dart';
import 'package:flutter/services.dart';

typedef ValueChanged<T> = void Function(T value);
typedef VoidCallback = void Function();
typedef GestureTapCallback = void Function();

class Forms {
  static border({
    TextEditingController? controller,
    String? hintText,
    String? title,
    TextInputType? keyboardType,
    TextAlign? textAlign,
    FocusNode? focusNode,
    bool? enabled,
    ValueChanged<String>? onChanged,
    Function? onSubmited,
    VoidCallback? onEditingComplete,
    GestureTapCallback? onTap,
    FormFieldValidator<String>? validator,
    Widget? prefixIcon,
    bool? suffixIcon,
    Widget? customSuffixIcon,
    bool? obsecureText,
    Color? color,
    Color? allColor,
    int? maxLength,
    int? height,
    Color? fillColor,
    required BuildContext context,
  }) {
    return Column(
      crossAxisAlignment: CrossAxisAlignment.start,
      children: [
        title == null
            ? const SizedBox()
            : Column(
                children: [
                  Padding(
                    padding: const EdgeInsets.only(left: 5),
                    child: Text(
                      title,
                      style: TextStyle(
                        color: allColor ?? color ?? Theme.of(context).textTheme.bodyText2!.color,
                        fontWeight: FontWeight.w600,
                        fontSize: Theme.of(context).textTheme.bodyText2!.fontSize,
                      ),
                    ),
                  ),
                  const SizedBox(
                    height: 8,
                  ),
                ],
              ),
        TextFormField(
          controller: controller,
          keyboardType: keyboardType,
          style: TextStyle(
            color: allColor ?? color ?? Theme.of(context).textTheme.bodyText2!.color,
            fontSize: Theme.of(context).textTheme.bodyText2!.fontSize,
          ),
          textAlign: textAlign ?? TextAlign.left,
          focusNode: focusNode,
          decoration: InputDecoration(
            hintText: hintText ?? '',
            counterText: "",
            hintStyle: TextStyle(
              color: allColor ?? color ?? Theme.of(context).textTheme.bodyText2!.color!.withOpacity(0.7),
              fontSize: Theme.of(context).textTheme.bodyText2!.fontSize,
            ),
            prefixIcon: prefixIcon,
            suffixIcon: customSuffixIcon ?? (suffixIcon != null && suffixIcon == true ? (controller != null && controller.text != '' ? Icon(Icons.close, color: Theme.of(context).textTheme.bodyText1!.color) : null) : null),
            contentPadding: const EdgeInsets.symmetric(horizontal: 16, vertical: 12),
            filled: true,
            fillColor: fillColor ?? Theme.of(context).scaffoldBackgroundColor,
            // focusColor: Colors.transparent,
            // hoverColor: Colors.transparent,
            border: OutlineInputBorder(
              borderRadius: BorderRadius.circular(5),
              borderSide: BorderSide(
                color: allColor ?? Theme.of(context).textTheme.bodyText2!.color!.withOpacity(0.7),
                width: 1.4,
              ),
            ),
            disabledBorder: OutlineInputBorder(
              borderRadius: BorderRadius.circular(5),
              borderSide: BorderSide(
                color: allColor ?? Theme.of(context).textTheme.bodyText2!.color!.withOpacity(0.4),
                width: 1.4,
              ),
            ),
            enabledBorder: OutlineInputBorder(
              borderRadius: BorderRadius.circular(5),
              borderSide: BorderSide(
                color: allColor ?? Theme.of(context).textTheme.bodyText2!.color!.withOpacity(0.7),
                width: 1.4,
              ),
            ),
            errorBorder: OutlineInputBorder(
              borderRadius: BorderRadius.circular(5),
              borderSide: BorderSide(
                color: allColor ?? color ?? const Color(0xffe45a4f),
                width: 1.4,
              ),
            ),
            focusedBorder: OutlineInputBorder(
              borderRadius: BorderRadius.circular(5),
              borderSide: BorderSide(
                color: allColor ?? color ?? Theme.of(context).textTheme.bodyText2!.color!,
                width: 1.4,
              ),
            ),
          ),
          enabled: enabled,
          onChanged: onChanged,
          validator: validator,
          maxLines: height ?? 1,
          maxLength: maxLength,
          onEditingComplete: () {
            final FocusScopeNode currentScope = FocusScope.of(context);
            if (!currentScope.hasPrimaryFocus && currentScope.hasFocus) {
              FocusManager.instance.primaryFocus!.unfocus();
            }
            SystemChrome.restoreSystemUIOverlays();
            if (onEditingComplete != null) {
              onEditingComplete();
            }
          },
          obscureText: obsecureText ?? false,
          onTap: () {
            Future.delayed(const Duration(milliseconds: 100), () {
              // SystemChrome.restoreSystemUIOverlays();
              // SystemChrome.restoreSystemUIOverlays();
            });
            if (onTap != null) onTap();
          },
        ),
      ],
    );
  }
}
